import { Chart, Colors } from "chart.js";
import chartDataLabels from 'chartjs-plugin-datalabels';
import doughnutLabel from 'chartjs-plugin-doughnutlabel-v3';
import { Vue } from "../..";

import 'chartkick/chart.js';

Chart.register(chartDataLabels);
Chart.register(doughnutLabel);
Chart.register(Colors);

import VueChartkick from 'vue-chartkick';
import { computed } from "vue";
Vue.use(VueChartkick);


const multipleDataSeries = (dataSeries) => {
    return dataSeries.map((value) => {
        let points = null;
        if (Array.isArray(value.points)) {
            points = value.points.reduce((prev, curr) => {
                return Object.assign(prev, { [curr.x]: curr.y });
            }, {});
        } else {
            points = value.points;
        }
        return { name: value.title, data: points };
    });
};

const singleDataSeries = (dataSeries) => dataSeries.reduce((prev, curr) => Object.assign(prev, { [curr.title]: curr.value }), {});

function useChart({ dataSeries, spec, multiple = true }) {
    const isDonut = [spec.styleClasses].flat().includes('donut');
    const { datalabels, centerLabel } = spec.plugins || {};
    const legend = spec.legend || { display: true };

    let series = null;
    if (multiple) {
        series = computed(() => multipleDataSeries(dataSeries));
    } else {
        series = computed(() => singleDataSeries(dataSeries));
    }

    let colors = undefined;
    if (spec.colors) colors = spec.colors;

    const options = {};
    options.plugins = {};
    if (legend) options.plugins.legend = legend;
    if (isDonut) options.cutout = '75%';
    if (!datalabels) {
        options.plugins.datalabels = { display: false };
    } else {
        options.plugins.datalabels = datalabels;
        options.plugins.datalabels.formatter = (value, context) => {
            const type = datalabels.formatType || 'value';

            switch (type) {
                case 'label':
                    return context.chart.data.labels[context.dataIndex];
                    break;

                case 'value':
                    return `${spec.prefix || ''}${value}${spec.suffix || ''}`;
                    break;
            }
        };
    }
    if (centerLabel) {
        options.plugins.doughnutLabel = {
            labels: centerLabel.labels
        };
    }

    return { series, colors, options };
}


export { useChart };